cfg_if::cfg_if! {
    if #[cfg(target_os = "macos")] {
        mod macos;
    } else if #[cfg(target_os = "linux")] {
        mod linux;
    } else if #[cfg(target_os = "windows")] {
        mod windows;
    } else {
        compile_error!("Unsupported OS");
    }
}

pub trait Vulnerability {
    /// Returns the name of the backend that has been chosen
    fn backend_name() -> &'static str;
    /// This should trigger the vulnerability, returns () on success
    fn trigger(&self, length: usize) -> crate::Result<()>;
    /// Validate the environment, this can detect environment errors before attempting
    /// to trigger the vulnerability
    fn validate_environment(&self) -> crate::Result<()>;
}
